#include "test.h"

void InitGpio(void) {
    EALLOW;
    /* Disable pin locks                                                        */
    GpioCtrlRegs.GPALOCK.all = 0x00000000;
    GpioCtrlRegs.GPBLOCK.all = 0x00000000;
    GpioCtrlRegs.GPCLOCK.all = 0x00000000;
    GpioCtrlRegs.GPDLOCK.all = 0x00000000;
    GpioCtrlRegs.GPELOCK.all = 0x00000000;
    GpioCtrlRegs.GPFLOCK.all = 0x00000000;
    EDIS;

    GPIO_SetupPinMux(CC_LD2_GPIO, GPIO_MUX_CPU1, 0);
    GPIO_SetupPinOptions(CC_LD2_GPIO, GPIO_OUTPUT, GPIO_PUSHPULL);
    GPIO_SetupPinMux(CC_LD3_GPIO, GPIO_MUX_CPU1, 0);
    GPIO_SetupPinOptions(CC_LD3_GPIO, GPIO_OUTPUT, GPIO_PUSHPULL);

    GPIO_WritePin(CC_LD2_GPIO, 1);
    GPIO_WritePin(CC_LD3_GPIO, 1);
}

void GPIO_SetupPinMux(Uint16 pin, Uint16 cpu, Uint16 peripheral) {
    volatile Uint32 *gpioBaseAddr;
    volatile Uint32 *mux, *gmux, *csel;
    Uint16 pin32, pin16, pin8;

    pin32 = pin % 32;
    pin16 = pin % 16;
    pin8 = pin % 8;
    gpioBaseAddr = (Uint32 *)&GpioCtrlRegs + ((pin / 32) * GPY_CTRL_OFFSET);

    /* Create pointers to the appropriate registers. This is a workaround
     * for the way GPIO registers are defined. The standard definition
     * in the header file makes it very easy to do named accesses of one
     * register or bit, but hard to do arbitrary numerical accesses. It's
     * easier to have an array of GPIO modules with identical registers,
     * including arrays for multi-register groups like GPyCSEL1-4. But
     * the header file doesn't define anything we can turn into an array,
     * so manual pointer arithmetic is used instead.                            */
    mux = gpioBaseAddr + GPYMUX + (pin32 / 16);
    gmux = gpioBaseAddr + GPYGMUX + (pin32 / 16);
    csel = gpioBaseAddr + GPYCSEL + (pin32 / 8);

    /* To change the muxing, set the peripheral mux to 0/GPIO first to avoid
     * glitches, then change the group mux, then set the peripheral mux to
     * its target value. Finally, set the CPU select. This procedure is
     * described in the TRM. Unfortunately, since we don't know the pin in
     * advance we can't hardcode a bitfield reference, so there's some tricky
     * bit twiddling here.                                                      */
    EALLOW;
    *mux &= ~(0x3UL << (2 * pin16));
    *gmux &= ~(0x3UL << (2 * pin16));
    *gmux |= ((Uint32)((peripheral >> 2) & 0x3UL) << (2 * pin16));
    *mux |= ((Uint32)(peripheral & 0x3UL) << (2 * pin16));
    *csel &= ~(0x3L << (4 * pin8));
    *csel |= ((Uint32)(cpu & 0x3L) << (4 * pin8));
    EDIS;

    /* WARNING: This code does not touch the analog mode select registers,
     * which are needed to give the USB module control of its IOs.              */
}

/* GPIO_SetupPinOptions - Setup up the GPIO input/output options for the
 *                        specified pin.
 *                        The flags are a 16-bit mask produced by ORing
 *                        together options.
 *                        For input pins, the valid flags are:
 *                        GPIO_PULLUP  Enable pull-up
 *                        GPIO_INVERT  Enable input polarity inversion
 *                        GPIO_SYNC    Synchronize the input latch to PLLSYSCLK
 *                                     (default-you don't need to specify this)
 *                        GPIO_QUAL3   Use 3-sample qualification
 *                        GPIO_QUAL6   Use 6-sample qualification
 *                        GPIO_ASYNC   Do not use synchronization or
 *                                     qualification
 *                        (Note: only one of SYNC, QUAL3, QUAL6, or ASYNC is
 *                         allowed)
 *
 *                        For output pins, the valid flags are:
 *                        GPIO_PUSHPULL    Normal output
 *                        GPIO_OPENDRAIN   Output in open drain mode
 *                        GPIO_PULLUP      If open drain enabled, also enable
 *                                         the pull-up and the input
 *                                         qualification flags
 *                                        (SYNC/QUAL3/QUAL6/SYNC) listed above.
 *
 *                        With no flags, the default input state is synchronous
 *                        with no pull-up or polarity inversion. The default
 *                        output state is the standard digital output.          */
void GPIO_SetupPinOptions(Uint16 pin, Uint16 output, Uint16 flags) {
    volatile Uint32 *gpioBaseAddr;
    volatile Uint32 *dir, *pud, *inv, *odr, *qsel;
    Uint32 pin32, pin16, pinMask, qual;

    pin32 = pin % 32;
    pin16 = pin % 16;
    pinMask = 1UL << pin32;
    gpioBaseAddr = (Uint32 *)&GpioCtrlRegs + ((pin / 32) * GPY_CTRL_OFFSET);

    /* Create pointers to the appropriate registers. This is a workaround
     * for the way GPIO registers are defined. The standard definition
     * in the header file makes it very easy to do named accesses of one
     * register or bit, but hard to do arbitrary numerical accesses. It's
     * easier to have an array of GPIO modules with identical registers,
     * including arrays for multi-register groups like GPyQSEL1-2. But
     * the header file doesn't define anything we can turn into an array,
     * so manual pointer arithmetic is used instead.                            */
    dir = gpioBaseAddr + GPYDIR;
    pud = gpioBaseAddr + GPYPUD;
    inv = gpioBaseAddr + GPYINV;
    odr = gpioBaseAddr + GPYODR;
    qsel = gpioBaseAddr + GPYQSEL + (pin32 / 16);

    EALLOW;

    *dir &= ~pinMask;
    if (output == 1) {
        /* Output, with optional open drain mode and pull-up                    */
        *dir |= pinMask;
        /* Enable open drain if necessary                                       */
        if (flags & GPIO_OPENDRAIN) *odr |= pinMask;
        else *odr &= ~pinMask;
        /* Enable pull-up if necessary. Open drain mode must be active.         */
        if (flags & (GPIO_OPENDRAIN | GPIO_PULLUP)) *pud &= ~pinMask;
        else *pud |= pinMask;
    }
    else {
        /* Input, with optional pull-up, qualification, and polarity inversion  */
        *dir &= ~pinMask;
        /* Enable pull-up if necessary                                          */
        if (flags & GPIO_PULLUP) *pud &= ~pinMask;
        else *pud |= pinMask;
        /* Invert polarity if necessary                                         */
        if (flags & GPIO_INVERT) *inv |= pinMask;
        else *inv &= ~pinMask;
    }

    /* Extract the qualification parameter and load it into the register.
     * This is also needed for open drain outputs, so we might as well do it
     * all the time.                                                            */
    qual = (flags & GPIO_ASYNC) / GPIO_QUAL3;
    *qsel &= ~(0x3L << (2 * pin16));
    if (qual != 0x0) *qsel |= (qual << (2 * pin16));

    EDIS;
}


Uint16 GPIO_ReadPin(Uint16 pin) {
    volatile Uint32 *gpioDataReg;
    Uint16 pinVal;

    gpioDataReg = (volatile Uint32 *)&GpioDataRegs + ((pin / 32) * GPY_DATA_OFFSET);
    pinVal = (gpioDataReg[GPYDAT] >> (pin % 32)) & 0x1;

    return pinVal;
}

void GPIO_WritePin(Uint16 pin, Uint16 outVal) {
    volatile Uint32 *gpioDataReg;
    Uint32 pinMask;

    gpioDataReg = (volatile Uint32 *)&GpioDataRegs + ((pin / 32) * GPY_DATA_OFFSET);
    pinMask = 1UL << (pin % 32);

    if (outVal == 0) gpioDataReg[GPYCLEAR] = pinMask;
    else gpioDataReg[GPYSET] = pinMask;
}
